<?php

namespace Drupal\fox\Plugin\FoxCommand;

/**
 * BROWSE fox command.
 *
 * @FoxCommand(
 *   id = "browse",
 *   label = @Translation("Browse fields for context record. Usage: BROWSE {*|fields}")
 * )
 */
class FoxCommandBrowse extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    $helper = $this->foxCommandsHelper();

    $entity_type = $variables['entity_type'] ?? NULL;
    $id = $variables['id'] ?? NULL;

    if (empty($entity_type)) {
      return [
        'error' => $this->t('Empty entity type. Use USE command.'),
      ];
    }

    if ($id === NULL) {
      return [
        'error' => $this->t('Empty ID. Use GOTO command.'),
      ];
    }

    $param = reset($params);
    if (empty($param)) {
      $param = $helper->entityKey($entity_type, 'id');
    }

    $entity = $helper->entityTypeManager()
      ->getStorage($entity_type)
      ->load($id);

    if (empty($entity)) {
      return [
        'error' => $this->t('Empty entity'),
      ];
    }

    $fieldsList = $entity->getFields();

    // Creating field list.
    if ($param === '*') {
      $fields = array_keys($fieldsList);
    }
    else {
      $fields = explode(',', $param);
    }

    $data = $row = $header = [];
    foreach ($fields as $field) {
      $name = trim($field);
      if (!$entity->hasField($name)) {
        continue;
      }
      $value = $entity->get($name)->getString();
      $row[] = $value;

      $header[] = $fieldsList[$name]->getFieldDefinition()->getLabel();
    }
    $data[] = $row;

    return [
      'message' => [
        'header' => $header,
        'data' => $data,
      ],
    ];
  }

}
